@@ -9,8 +9,11 @@ import re
 import csv
 import sys
 import errno
+import codecs
+import signal
 import urllib
 import datetime
+import optparse
 import tempfile
 import ConfigParser
 
@@ -238,7 +241,7 @@ class TimeWindow(object):
                 f = self.filename
                 f.seek(0)
             else:
-                f = open(self.filename)
+                f = codecs.open(self.filename, encoding='UTF-8')
         except IOError:
             return
         line = ''
@@ -442,7 +445,7 @@ class TimeWindow(object):
         if title_row:
             writer.writerow(["task", "time (minutes)"])
         work, slack = self.grouped_entries()
-        work = [(entry, as_minutes(duration))
+        work = [(entry.encode('UTF-8'), as_minutes(duration))
                 for start, entry, duration in work
                 if duration] # skip empty "arrival" entries
         work.sort()
@@ -805,7 +808,7 @@ class TimeLog(object):
 
     def raw_append(self, line):
         """Append a line to the time log file."""
-        f = open(self.filename, "a")
+        f = codecs.open(self.filename, "a", encoding='UTF-8')
         if self.need_space:
             self.need_space = False
             print >> f
@@ -976,6 +979,13 @@ class Settings(object):
 
     report_style = 'plain'
 
+    def get_config_dir(self):
+        envar_home = os.environ.get('GTIMELOG_HOME')
+        return os.path.expanduser(envar_home if envar_home else default_home)
+
+    def get_config_file(self):
+        return os.path.join(self.get_config_dir(), 'gtimelogrc')
+
     def _config(self):
         config = ConfigParser.RawConfigParser()
         config.add_section('gtimelog')
@@ -1044,11 +1054,14 @@ class IconChooser:
         # not necessarily the case!  this logic works for, say,
         # Ambiance/Radiance, but it gets New Wave and Dark Room wrong.
         if toolkit == 'gi':
-            # XXX: need to figure out how to do that with gi
-            return icon_file_bright
-
-        menu_text_color = gtk.MenuBar().rc_get_style().text[gtk.STATE_NORMAL]
-        if menu_text_color.value >= 0.5:
+            style = gtk.MenuBar().get_style_context()
+            color = style.get_color(gtk.StateFlags.NORMAL)
+            value = (color.red + color.green + color.blue) / 3
+        else:
+            style = gtk.MenuBar().rc_get_style()
+            color = style.text[gtk.STATE_NORMAL]
+            value = color.value
+        if value >= 0.5:
             return icon_file_bright
         else:
             return icon_file_dark
@@ -1070,7 +1083,9 @@ class SimpleStatusIcon(IconChooser):
         self.icon.connect('activate', self.on_activate)
         self.icon.connect('popup-menu', self.on_popup_menu)
         self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set)
+            'style-set', self.on_style_set) # Gtk+ 2
+        self.gtimelog_window.main_window.connect(
+            'style-updated', self.on_style_set) # Gtk+ 3
         gobject.timeout_add(1000, self.tick)
         self.gtimelog_window.entry_watchers.append(self.entry_added)
         self.gtimelog_window.tray_icon = self
@@ -1138,7 +1153,9 @@ class AppIndicator(IconChooser):
         self.indicator.set_menu(gtimelog_window.app_indicator_menu)
         self.gtimelog_window.tray_icon = self
         self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set)
+            'style-set', self.on_style_set) # Gtk+ 2
+        self.gtimelog_window.main_window.connect(
+            'style-updated', self.on_style_set) # Gtk+ 3
 
     def available(self):
         """Is the icon supported by this system?
@@ -1182,7 +1199,9 @@ class OldTrayIcon(IconChooser):
         self.tick(force_update=True)
         self.trayicon.show_all()
         self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set)
+            'style-set', self.on_style_set) # Gtk+ 2
+        self.gtimelog_window.main_window.connect(
+            'style-updated', self.on_style_set) # Gtk+ 3
         tray_icon_popup_menu = gtimelog_window.tray_icon_popup_menu
         self.eventbox.connect_object(
             'button-press-event', self.on_press, tray_icon_popup_menu)
@@ -1782,7 +1801,7 @@ class MainWindow:
     def mail(self, write_draft):
         """Send an email."""
         draftfn = tempfile.mktemp(suffix='gtimelog') # XXX unsafe!
-        with open(draftfn, 'w') as draft:
+        with codecs.open(draftfn, 'w', encoding='UTF-8') as draft:
             write_draft(draft, self.settings.email, self.settings.name)
         self.spawn(self.settings.mailer, draftfn)
         # XXX rm draftfn when done -- but how?
@@ -1909,6 +1928,8 @@ class MainWindow:
     def add_entry(self, widget, data=None):
         """Add the task entry to the log."""
         entry = self.task_entry.get_text()
+        if not isinstance(entry, unicode):
+            entry = unicode(entry, 'UTF-8')
 
         now = None
         date_match = re.match(r'(\d\d):(\d\d)\s+', entry)
@@ -1993,17 +2014,35 @@ if dbus:
         def Present(self):
             self.main_window.on_show_activate()
 
+        @dbus.service.method(INTERFACE)
+        def Quit(self):
+            gtk.main_quit()
+
 
 def main():
     """Run the program."""
-    # XXX convert argument parsing to argparse.
-    if len(sys.argv) > 1 and sys.argv[1] == '--sample-config':
+    parser = optparse.OptionParser(usage='%prog [options]')
+    parser.add_option('--sample-config', action='store_true',
+        help="write a sample configuration file to 'gtimelogrc.sample'")
+    parser.add_option('--ignore-dbus', action='store_true',
+        help="do not check if GTimeLog is already running")
+    parser.add_option('--replace', action='store_true',
+        help="replace the already running GTimeLog instance")
+    parser.add_option('--toggle', action='store_true',
+        help="show/hide the GTimeLog window if already running")
+    parser.add_option('--tray', action='store_true',
+        help="start minimized")
+
+    opts, args = parser.parse_args()
+
+    if opts.sample_config:
         settings = Settings()
         settings.save("gtimelogrc.sample")
         print "Sample configuration file written to gtimelogrc.sample"
+        print "Edit it and save as %s" % settings.get_config_file()
         return
 
-    if '--ignore-dbus' in sys.argv:
+    if opts.ignore_dbus:
         global dbus
         dbus = None
 
@@ -2016,28 +2055,37 @@ def main():
         try:
             session_bus = dbus.SessionBus()
             dbus_service = session_bus.get_object(SERVICE, OBJECT_PATH)
-            if '--toggle' in sys.argv:
+            if opts.replace:
+                print 'gtimelog: Telling the already-running instance to quit'
+                dbus_service.Quit()
+            elif opts.toggle:
                 dbus_service.ToggleFocus()
-                print 'Already running, toggling visibility'
-            elif '--tray' in sys.argv:
-                print 'Already running, not doing anything'
+                print 'gtimelog: Already running, toggling visibility'
+                sys.exit()
+            elif opts.tray:
+                print 'gtimelog: Already running, not doing anything'
+                sys.exit()
             else:
                 dbus_service.Present()
-                print 'Already running, presenting main window'
-            sys.exit()
-        except dbus.DBusException:
-            pass
+                print 'gtimelog: Already running, presenting main window'
+                sys.exit()
+        except dbus.DBusException, e:
+            if e.get_dbus_name() == 'org.freedesktop.DBus.Error.ServiceUnknown':
+                # gtimelog is not running: that's fine and not an error at all
+                pass
+            else:
+                sys.exit('gtimelog: %s' % e)
 
-    envar_home = os.environ.get('GTIMELOG_HOME')
-    configdir = os.path.expanduser(envar_home if envar_home else default_home)
+    settings = Settings()
+    configdir = settings.get_config_dir()
     try:
         # Create it if it doesn't exist.
         os.makedirs(configdir)
     except OSError as error:
         if error.errno != errno.EEXIST:
+            # XXX: not the most friendly way of error reporting for a GUI app
             raise
-    settings = Settings()
-    settings_file = os.path.join(configdir, 'gtimelogrc')
+    settings_file = settings.get_config_file()
     if not os.path.exists(settings_file):
         settings.save(settings_file)
     else:
@@ -2063,12 +2111,15 @@ def main():
             if tray_icon.available():
                 start_in_tray = (settings.start_in_tray
                                  if settings.start_in_tray
-                                 else ('--tray' in sys.argv))
+                                 else opts.tray)
                 break # found one that works
     if not start_in_tray:
         main_window.on_show_activate()
     if dbus:
         service = Service(main_window)
+    # This is needed to make ^C terminate gtimelog when we're using
+    # gobject-introspection.
+    signal.signal(signal.SIGINT, signal.SIG_DFL)
     try:
         gtk.main()
     except KeyboardInterrupt:
